/* usbtenkiget: A command-line tool for reading USBTenki sensors.
 * Copyright (C) 2007-2014  Raphael Assenat <raph@raphnet.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef _usbtenki_h__
#define _usbtenki_h__

//#include <usb.h>

#include "usbtenki_units.h"

#ifdef __cplusplus
extern "C" {
#endif



struct USBTenki_info {
	char str_prodname[256];
	char str_serial[256];
	int major, minor;
};

struct USBTenki_list_ctx;

#define USBTENKI_CHN_STATUS_UNDEFINED			0
#define USBTENKI_CHN_STATUS_VALID				1
#define USBTENKI_CHN_STATUS_SATURATED			2
#define USBTENKI_CHN_STATUS_SENSOR_ERROR		3
#define USBTENKI_CHN_STATUS_PROBE_DISCONNECTED	4
#define USBTENKI_CHN_STATUS_OUT_OF_RANGE		5
#define USBTENKI_CHN_STATUS_INVALID_DATA		6
#define USBTENKI_CHN_STATUS_SENSOR_SHORTED		7

// Undefined / unset points will be all NaN
//
// usbtenki_isCalpointSet() can be used to check.
//
struct USBTenki_calpoint {
	double sensed_value;
	double real_value;
};

#define USBTENKI_MAX_CALPOINTS	3

struct USBTenki_channel {
	int channel_id;
	int chip_id;
	unsigned char status;
	unsigned char raw_data[32];
	int raw_length;
	double raw_value; // Not always used.
	double converted_data; // May be compensated by user-calibration
	double pre_calibration_data; // Copy of converted_data prior to user-calibration
	int converted_unit;
	// number of user-calibration points returned by the
	// hardware. Some may be undefied.
	int num_calpoints;
	struct USBTenki_calpoint calpoints[USBTENKI_MAX_CALPOINTS];

	// When non-zero, virtual channels will use the value from pre_calibration_data
	// when using this channel.
	//
	// This flag can also be used by the application to select between calibrated and
	// non-calibrated values for regular channels. Unless calibration is disabled globally,
	// the converted_data field will still contain the calibrated value for channels
	// with calibration points.
	int use_precal_data;
};

// Note: usbtenki_channelValue() can be used to retrive the channel value (it selects
// between converted_data and pre_calibration_data) according to use_precal_data.

struct USBTenki_unitPreferences {
	int temperature;
	int pressure;
	int frequency;
	int voltage;
	int current;
	int power;
	int length;
	int concentration; // PPM, PPB, PERCENT
};

#ifndef ARRAY_SIZE
#define ARRAY_SIZE(a)	(sizeof(a)/sizeof(a[0]))
#endif

#define USBTENKI_FLAG_NO_HUMIDEX_RANGE				1
#define USBTENKI_FLAG_NO_HEAT_INDEX_RANGE			2
#define USBTENKI_FLAG_USE_OLD_SHT75_COMPENSATION	4
#define USBTENKI_FLAG_VERBOSE						8
#define USBTENKI_FLAG_NO_USER_CALIBRATION			16

typedef void* USBTenki_dev_handle; // Cast from usb_dev_handle
typedef void* USBTenki_device; // Cast from usb_device

#define USBTENKI_DEV_TO_USB_DEVICE(d) ((struct usb_device*)(d))

int usbtenki_init(void);
void unsbtenki_shutdown(void);

struct USBTenki_list_ctx *usbtenki_allocListCtx(void);
void usbtenki_freeListCtx(struct USBTenki_list_ctx *ctx);

USBTenki_device usbtenki_listDevices(struct USBTenki_info *info,
										struct USBTenki_list_ctx *ctx);

USBTenki_dev_handle usbtenki_openDevice(USBTenki_device tdev);
USBTenki_dev_handle usbtenki_openBySerial(const char *serial, struct USBTenki_info *info);

void usbtenki_closeDevice(USBTenki_dev_handle hdl);


int usbtenki_command(USBTenki_dev_handle hdl, unsigned char cmd,
										unsigned int id, unsigned char *dst, int dst_max_size);
int usbtenki_getRaw(USBTenki_dev_handle hdl, int id, unsigned char *dst, int dst_max_size);

// dst must be sized to hold USBTENKI_MAX_CALPOINTS elements of struct USBTenki_calpoint
int usbtenki_getChannelCalibrationPoints(USBTenki_dev_handle hdl, int id, struct USBTenki_calpoint *dst);
int usbtenki_setUserCalibration(USBTenki_dev_handle hdl, int channel_id, int point_id, const struct USBTenki_calpoint *pt);
int usbtenki_getUserCalibration(USBTenki_dev_handle hdl, int id, unsigned char *dst, int dst_max_size);
int usbtenki_getCalibration(USBTenki_dev_handle hdl, int id, unsigned char *dst, int dst_max_size);
int usbtenki_getChipID(USBTenki_dev_handle hdl, int id);
int usbtenki_getNumChannels(USBTenki_dev_handle hdl);

int usbtenki_set_dxc120_asc(USBTenki_dev_handle hdl, int value);
int usbtenki_set_dxc120_frc(USBTenki_dev_handle hdl, int value);
int usbtenki_get_dxc120_asc(USBTenki_dev_handle hdl);

int usbtenki_set_dxc220_kz(USBTenki_dev_handle hdl, int value);

int usbtenki_convertRaw(struct USBTenki_channel *chn, unsigned long flags, unsigned char *caldata, int caldata_len);

int usbtenki_listChannels(USBTenki_dev_handle hdl, struct USBTenki_channel *dstArray, int arr_size);

void usbtenki_set_seaLevelStandardPressure(double slp_P); // in Pascals (default: 101325)
/**
 * \brief Add virtual channels based on previously listed ones (see usbtenki_listChannels)
 * \param chnArray array to hold the channels
 * \param num_channels The number of channels available in the arrary
 * \param arr_size The maximum number of channels that can be held by the array.
 */
int usbtenki_addVirtualChannels(struct USBTenki_channel *chnArray, int *num_channels, int arr_size);
int usbtenki_processVirtualChannels(USBTenki_dev_handle hdl, struct USBTenki_channel channels[], int num_channels, unsigned long flags);
int usbtenki_processSomeVirtualChannels(USBTenki_dev_handle hdl, struct USBTenki_channel channels[], int num_channels, const int requested_channels[], int num_req_chns, unsigned long flags);

int usbtenki_readChannelList(USBTenki_dev_handle hdl, const int channel_ids[], int num, struct USBTenki_channel *dst, int dst_total, int num_attempts, unsigned long flags);
int usbtenki_readChannel(USBTenki_dev_handle hdl, struct USBTenki_channel *chn, unsigned long flags);

double usbtenki_convertTemperature(double value, int src_unit, int dst_unit);
double usbtenki_convertPressure(double value, int src_unit, int dst_unit);
double usbtenki_convertFrequency(double value, int src_unit, int dst_unit);
double usbtenki_convertVoltage(double value, int src_unit, int dst_unit);
double usbtenki_convertCurrent(double value, int src_unit, int dst_unit);
double usbtenki_convertPower(double value, int src_unit, int dst_unit);
double usbtenki_convertConcentration(double value, int src_unit, int dst_unit);
const char *chipToString(int id);
const char *chipToShortString(int id);
const char *unitToString(int unit, int no_fancy_chars);
const char *thermocoupleTypeToString(int type);
int thermocoupleStringToType(const char *type);

int usbtenki_getChipUnit(int chip_id);
int usbtenki_getPreferredUnit(int source_unit, const struct USBTenki_unitPreferences *units);
void usbtenki_convertUnits(struct USBTenki_channel *chn, const struct USBTenki_unitPreferences *units);

const char *usbtenki_getChannelStatusString(const struct USBTenki_channel *chn);
const char *usbtenki_getChannelStatusStringNoSpaces(const struct USBTenki_channel *chn);

int usbtenki_isCalpointSet(struct USBTenki_calpoint *cp);
int usbtenki_convertCalpointsUnits(struct USBTenki_calpoint *calpoints, int num_calpoints, int source_units, int target_units);
double usbtenki_channelValue(struct USBTenki_channel *chn);


#ifdef __cplusplus
}
#endif

#endif // _usbtenki_h__
